/*
--
-- This script updates the stored procedure GetExpiredItemExclusive in Sitecore Sessions database.
--
*/

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

BEGIN TRANSACTION

IF(OBJECTPROPERTY(OBJECT_ID(N'[dbo].[GetExpiredItemExclusive]'), N'IsProcedure') = 1)
  DROP PROCEDURE [dbo].[GetExpiredItemExclusive]
GO

CREATE PROCEDURE [GetExpiredItemExclusive]
  @application UNIQUEIDENTIFIER,
  @id NVARCHAR(88) OUTPUT,
  @lockTimestamp DATETIME,
  @lockCookie CHAR(32)
AS
BEGIN

  SET NOCOUNT ON;

  DECLARE @now DATETIME = GETUTCDATE();
  DECLARE @result INTEGER = 0;

  UPDATE
    [Sessions]
  SET
    @id = [Id],
    [Expires] = DATEADD( n, [Timeout], @lockTimestamp ),
    [LockTimestamp] = @lockTimestamp,
    [LockCookie] = @lockCookie
  WHERE
    ([Id] = (SELECT TOP 1 [Id] FROM [Sessions] with(updlock) WHERE ([Application] = @application) AND ([Expires] < @now) ORDER BY [Expires]));

  IF( @@ROWCOUNT <> 0 )
  BEGIN
    SET @result = 1;
  END;

  SELECT
    CASE @result WHEN 1 THEN [Item] ELSE NULL END
  FROM
    [Sessions]
  WHERE
    ([Application] = @application) AND
    ([Id] = @id);

  RETURN @result;

END

GO

COMMIT TRANSACTION

GO